@@ -3,8 +3,6 @@
 
 #include <selinux/selinux.h>
 
-#define _DEFCONTEXT_PATH "/etc/security/default_contexts"
-#define _FAILSAFECONTEXT_PATH "/etc/security/failsafe_context"
 #define SELINUX_DEFAULTUSER "user_u"
 
 /* Get an ordered list of authorized security contexts for a user session
@@ -29,6 +27,14 @@ extern int get_default_context(const char* user,
 			       security_context_t fromcon,
 			       security_context_t *newcon);
 
+/* Same as get_default_context, but only return a context
+   that has the specified role.  If no reachable context exists
+   for the user with that role, then return -1. */
+int get_default_context_with_role(const char* user, 
+				  const char *role,
+				  security_context_t fromcon,
+				  security_context_t *newcon);
+
 /* Given a list of authorized security contexts for the user, 
    query the user to select one and set *newcon to refer to it.
    Caller must free via freecon.
