@@ -5,7 +5,8 @@
 #ifndef _SELINUX_GET_DEFAULT_TYPE_H_
 #define _SELINUX_GET_DEFAULT_TYPE_H_
 
-#define _DEFTYPE_PATH "/etc/security/default_type"
+/* Return path to default type file. */
+const char *selinux_default_type_path(void);
 
 /* Get the default type (domain) for 'role' and set 'type' to refer to it.
    Caller must free via free().
