@@ -2,6 +2,7 @@
 #define _SELINUX_H_
 
 #include <sys/types.h>
+#include <stdarg.h>
 
 /* Return 1 if we are running on a SELinux kernel, or 0 if not or -1 if we get an error. */
 extern int is_selinux_enabled(void);
@@ -72,12 +73,6 @@ extern int getpeercon(int fd, security_context_t *con);
 
 /* Wrappers for the selinuxfs (policy) API. */
 
-/* Mount point for selinuxfs. */
-#define SELINUXMNT "/selinux/"
-
-/* Default pathname for policy configuration, without version number. */
-#define SELINUXPOLICY "/etc/security/selinux/policy"
-
 typedef unsigned int access_vector_t;
 typedef unsigned short security_class_t;
 
@@ -119,6 +114,11 @@ extern int security_compute_user(security_context_t scon,
 /* Load a policy configuration. */
 extern int security_load_policy(void *data, size_t len);
 
+/* Load policy boolean settings.
+   Path may be NULL, in which case the booleans are loaded from
+   the active policy boolean configuration file. */
+extern int security_load_booleans(char *path);
+
 /* Check the validity of a security context. */
 extern int security_check_context(security_context_t con);
 
@@ -152,20 +152,72 @@ extern int security_commit_booleans(void);
 /* Common helpers */
 
 /* Return the security class value for a given class name. */
-security_class_t string_to_security_class(const char *name);
+extern security_class_t string_to_security_class(const char *name);
 
 /* Return an access vector for a given class and permission name. */
-access_vector_t string_to_av_perm(security_class_t tclass, const char *name);
+extern access_vector_t string_to_av_perm(security_class_t tclass, const char *name);
 
 /* Display an access vector in a string representation. */
-void print_access_vector(security_class_t tclass, access_vector_t av);
+extern void print_access_vector(security_class_t tclass, access_vector_t av);
+
+/* Set the function used by matchpathcon when displaying
+   errors about the file_contexts configuration.  If not set,
+   then this defaults to fprintf(stderr, fmt, ...). */
+void set_matchpathcon_printf(void (*f)(const char *fmt, ...));
 
 /* Match the specified pathname and mode against the file contexts
    configuration and set *con to refer to the resulting context.
    'mode' can be 0 to disable mode matching.
    Caller must free via freecon. */
-int matchpathcon(const char *path,
+extern int matchpathcon(const char *path,
 		 mode_t mode,
 		 security_context_t *con);
 
+/* Match the specified media and against the media contexts 
+   configuration and set *con to refer to the resulting context.
+   Caller must free con via freecon. */
+extern int matchmediacon(const char *path,
+		 security_context_t *con);
+
+/*
+  selinux_getenforcemode reads the /etc/selinux/config file and determines 
+  whether the machine should be started in enforcing (1), permissive (0) or 
+  disabled (-1) mode.
+ */
+extern int selinux_getenforcemode(int *enforce);
+
+/*
+  selinux_policy_root reads the /etc/selinux/config file and returns 
+  the directory path under which the compiled policy file and context 
+  configuration files exist.
+ */
+extern const char *selinux_policy_root(void);
+
+/* These functions return the paths to specific files under the 
+   policy root directory. */
+extern const char *selinux_binary_policy_path(void);
+extern const char *selinux_failsafe_context_path(void);
+extern const char *selinux_removable_context_path(void);
+extern const char *selinux_default_context_path(void);
+extern const char *selinux_user_contexts_path(void);
+extern const char *selinux_file_context_path(void);
+extern const char *selinux_media_context_path(void);
+extern const char *selinux_contexts_path(void);
+extern const char *selinux_booleans_path(void);
+
+/* Check a permission in the passwd class.
+   Return 0 if granted or -1 otherwise. */
+extern int checkPasswdAccess(access_vector_t requested);
+
+/* Set the path to the selinuxfs mount point explicitly.
+   Normally, this is determined automatically during libselinux 
+   initialization, but this is not always possible, e.g. for /sbin/init
+   which performs the initial mount of selinuxfs. */
+void set_selinuxmnt(char *mnt);
+
+/* Execute a helper for rpm in an appropriate security context. */
+extern int rpm_execcon(unsigned int verified, 
+		       const char *filename, 
+		       char *const argv[], char *const envp[]);
+
 #endif
