@@ -7,7 +7,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <selinux/selinux.h>
+#include "selinux_internal.h"
 #include <selinux/avc.h>
 #include "avc_sidtab.h"
 #include "avc_internal.h"
