@@ -1,5 +1,8 @@
 /*
  * Author: Karl MacMillan <kmacmillan@tresys.com>
+ *
+ * Modified:  
+ *   Dan Walsh <dwalsh@redhat.com> - Added security_load_booleans().
  */
 
 #include <sys/types.h>
@@ -13,8 +16,10 @@
 #include <unistd.h>
 #include <fnmatch.h>
 #include <limits.h>
+#include <ctype.h>
+#include <errno.h>
 
-#include <selinux/selinux.h>
+#include "selinux_internal.h"
 #include "policy.h"
 
 #define SELINUX_BOOL_DIR "/booleans/"
@@ -40,6 +45,7 @@ int security_get_boolean_names(char ***names, int *len)
 	char **n;
 
 	assert(len);
+	assert(selinux_mnt);
 
 	snprintf(path, sizeof path, "%s%s", selinux_mnt, SELINUX_BOOL_DIR);
 	*len = scandir(path, &namelist, &filename_select,
@@ -88,6 +94,8 @@ static int get_bool_value(const char *name, char **buf)
 	int fd, len;
 	char *fname = NULL;
 
+	assert(selinux_mnt);
+
 	*buf = (char*)malloc(sizeof(char) * (STRBUF_SIZE + 1));
 	if (!*buf)
 		goto out;
@@ -156,6 +164,8 @@ int security_set_boolean(const char *name, int value)
 	int fd, ret, len;
 	char buf[2], *fname;
 
+	assert(selinux_mnt);
+
 	len = strlen(name) + strlen(selinux_mnt) + sizeof(SELINUX_BOOL_DIR);
 	fname = (char*)malloc(sizeof(char) * len);
 	if (!fname)
@@ -183,6 +193,7 @@ out:
 	else
 		return -1;
 }
+hidden_def(security_set_boolean)
 
 int security_commit_booleans(void)
 {
@@ -190,6 +201,8 @@ int security_commit_booleans(void)
 	char buf[2];
 	char path[PATH_MAX];
 
+	assert(selinux_mnt);
+
 	snprintf(path, sizeof path, "%s/commit_pending_bools", selinux_mnt);
 	fd = open(path, O_WRONLY);
 	if (fd < 0)
@@ -206,4 +219,72 @@ int security_commit_booleans(void)
 	else
 		return -1;
 }
+hidden_def(security_commit_booleans)
 
+static char *strtrim(char *dest, char *source, int size) {
+	int i=0;
+	char *ptr=source;
+	i=0;
+	while(isspace(*ptr) && i < size) {
+		ptr++;
+		i++;
+	}
+	strncpy(dest,ptr,size);
+	for(i=strlen(dest)-1; i> 0; i--) {
+		if (!isspace(dest[i])) break;
+	}
+	dest[i+1]='\0';
+	return dest;
+}
+
+int security_load_booleans(char *path) {
+	FILE *boolf;
+	char buffer[BUFSIZ];
+	char name[BUFSIZ];
+	char name1[BUFSIZ];
+	int val;
+	int errors=0;
+
+	boolf = fopen(path ? path : selinux_booleans_path(),"r");
+	if (boolf == NULL) 
+		return -1;
+
+        while (fgets_unlocked(buffer, sizeof(buffer), boolf)) {
+		char *ptr;
+		char *tok=strtok_r(buffer,"=",&ptr);
+		if (tok) {
+			strncpy(name1,tok, BUFSIZ-1);
+			strtrim(name,name1,BUFSIZ-1);
+			if ( name[0]=='#' ) continue;
+			tok=strtok_r(NULL,"\0",&ptr);
+			if (tok) {
+				while (isspace(*tok)) tok++;
+				val = -1;
+				if (isdigit(tok[0]))
+					val=atoi(tok);
+				else if (!strncmp(tok, "true", sizeof("true")-1))
+					val = 1;
+				else if (!strncmp(tok, "false", sizeof("false")-1))
+					val = 0;
+				if (val != 0 && val != 1) {
+					fprintf(stderr,"illegal value for boolean %s=%s\n", name, tok);
+					errors++;
+					continue;
+				}
+
+				if (security_set_boolean(name, val) < 0) {
+					fprintf(stderr,"error setting boolean %s to value %d \n", name, val);
+					errors++;
+				}
+			}
+		}
+	}
+	fclose(boolf);
+
+	if (security_commit_booleans() < 0)
+		return -1;
+
+	if (errors)
+		errno = EINVAL;
+	return errors ? -1 : 0;
+}
