@@ -1,4 +1,4 @@
-#include <selinux/context.h>
+#include "context_internal.h"
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,12 +35,23 @@ context_new(const char *str)
                 case '\n': case '\t': case '\r': case ' ': goto err; /* sanity check */
                 }
         }
-        if ( count != 2 && count != 3 ) { /* might not have a range */
+	/*
+	 * Could be anywhere from 2 - 5
+	 * e.g user:role:type to user:role:type:sens1:cata-sens2:catb
+	 */
+        if ( count < 2 || count > 5 ) { /* might not have a range */
                 goto err;
         }
+
         n->component[3] = 0;
         for ( i = 0, tok = str; *tok; i++ ) {
-                for ( p = tok; *p && *p != ':'; p++ ) { /* empty */ }
+		if (i<3)
+                	for ( p = tok; *p && *p != ':'; p++ ) { /* empty */ }
+		else
+		{
+			/* MLS range is one component */
+                	for ( p = tok; *p; p++ ) { /* empty */ }
+		}
                 n->component[i] = (char*) malloc(p-tok+1);
 		if (n->component[i] == 0)
 		  goto err;
@@ -53,6 +64,7 @@ context_new(const char *str)
         context_free(result);
         return 0;
 }
+hidden_def(context_new)
 
 static void 
 conditional_free(char** v)
@@ -84,6 +96,7 @@ context_free(context_t context)
                 free(context);
         }
 }
+hidden_def(context_free)
 
 /*
  * Return a pointer to the string value of the context.
@@ -103,18 +116,19 @@ context_str(context_t context)
         }
         n->current_str = malloc(total);
         if ( n->current_str != 0 ) {
-                strcpy(n->current_str,n->component[0]);
-                strcat(n->current_str,":");
-                strcat(n->current_str,n->component[1]);
-                strcat(n->current_str,":");
-                strcat(n->current_str,n->component[2]);
+                char *cp = stpcpy(n->current_str,n->component[0]);
+                *cp++ = ':';
+                cp = stpcpy(cp,n->component[1]);
+                *cp++ = ':';
+                cp = stpcpy(cp,n->component[2]);
                 if ( n->component[3] ) {
-                        strcat(n->current_str,":");
-                        strcat(n->current_str,n->component[3]);
+                        *cp++ = ':';
+                        strcpy(cp,n->component[3]);
                 }
         }
         return n->current_str;
 }
+hidden_def(context_str)
 
 /* Returns nonzero iff failed */
 
@@ -125,7 +139,7 @@ static int set_comp(context_private_t* n,int index, const char *str)
         if ( !t ) { return 1; }
         for ( p = str; *p; p++ ) {
                 if ( *p == '\t' || *p == ' ' || *p == '\n' || *p == '\r' ||
-                     *p == ':' ) {
+                     (*p == ':' && index != COMP_RANGE) ) {
                         free(t);
                         return 1;
                 }
@@ -152,7 +166,8 @@ def_get(role,COMP_ROLE)
 int context_ ## name ## _set(context_t context, const char* str) \
 { \
         return set_comp(context->ptr,tag,str);\
-}
+} \
+hidden_def(context_ ## name ## _set)
 
 def_set(type,COMP_TYPE)
 def_set(role,COMP_ROLE)
@@ -167,6 +182,7 @@ int context_range_set(context_t context,const char* str)
                 return set_comp(n,COMP_RANGE,str);
         }
 }
+hidden_def(context_range_set)
 
 #ifdef L1TEST
 
@@ -175,12 +191,12 @@ int context_range_set(context_t context,const char* str)
 main()
 {
         context_t c1,c2;
-        c1 = context_new("user:role:type:levellow-levelhigh");
+        c1 = context_new("user:role:type:levellow:cat1,cat2-levelhigh:cat3");
         c2 = context_new("user2:role2:type2");
 
         /* see if strings come back unchanged */
 
-        if ( strcmp(context_str(c1),"user:role:type:levellow-levelhigh") ) {
+        if ( strcmp(context_str(c1),"user:role:type:levellow:cat1,cat2-levelhigh:cat3") ) {
                 test_fail("context_str c1");
         }
         if ( strcmp(context_str(c2),"user2:role2:type2") ) {
@@ -198,7 +214,7 @@ main()
         if ( strcmp(context_type_get(c1),"type") ) {
                 test_fail("context_type_get(c1)");
         }
-        if ( strcmp(context_range_get(c1),"levellow-levelhigh" ) ) {
+        if ( strcmp(context_range_get(c1),"levellow:cat1,cat2-levelhigh:cat3" ) ) {
                 test_fail("context_range_get(c1)");
         }
         if ( strcmp(context_role_get(c2),"role2") ) {
@@ -220,8 +236,8 @@ main()
              strcmp(context_type_get(c1),"newtype1") ) {
                 test_fail("context_type_set(c1)");
         }
-        if ( context_range_set(c1,"newrange1") ||
-             strcmp(context_range_get(c1),"newrange1") ) {
+        if ( context_range_set(c1,"newrange1:newcat1") ||
+             strcmp(context_range_get(c1),"newrange1:newcat1") ) {
                 test_fail("context_range_set(c1)");
         }
         if ( context_role_set(c1,"newrole1") ||
@@ -239,15 +255,15 @@ main()
                 test_fail("context_type_set with whitespace");
         }
 
-        /* check trying to set with a colon */
+        /* check trying to set non-range component with a colon */
 
-        if ( !context_range_set(c1,"newrange:1") ) {
+        if ( !context_type_set(c1,"newrole:newtype") ) {
                 test_fail("context_type_set with colon");
         }
 
         /* check new value */
 
-        if ( strcmp(context_str(c1),"newuser1:newrole1:newtype1:newrange1") ) {
+        if ( strcmp(context_str(c1),"newuser1:newrole1:newtype1:newrange1:newcat1") ) {
                 test_fail("second context_str(c1)");
         }
 
