@@ -5,13 +5,52 @@
 #include <string.h>
 #include <ctype.h>
 #include <pwd.h>
-#include <selinux/selinux.h>
+#include "selinux_internal.h"
 #include <selinux/context.h>
-#include <selinux/get_context_list.h>
+#include "get_context_list_internal.h"
 
 #define USERPRIORITY 1
 #define SYSTEMPRIORITY 2
 
+int get_default_context_with_role(const char* user, 
+				  const char *role,
+				  security_context_t fromcon,
+				  security_context_t *newcon)
+{
+    security_context_t *conary;
+    char **ptr;
+    context_t con;
+    const char *role2;
+    int rc;
+
+    rc = get_ordered_context_list(user, fromcon, &conary);
+    if (rc <= 0)
+	    return -1;
+
+    for (ptr = conary; *ptr; ptr++) {
+	    con = context_new(*ptr);
+	    if (!con)
+		    continue;
+	    role2 = context_role_get(con);
+	    if (role2 && !strcmp(role, role2)) {
+		    context_free(con);
+		    break;
+	    }
+	    context_free(con);
+    }
+
+    rc = -1;
+    if (!(*ptr))
+	    goto out;
+    *newcon = strdup(*ptr);
+    if (!(*newcon))
+	    goto out;
+    rc = 0;
+out:
+    freeconary(conary);
+    return rc;
+}
+
 int get_default_context(const char* user, 
 			security_context_t fromcon,
 			security_context_t *newcon)
@@ -45,7 +84,7 @@ static int find_line (FILE *infile, security_context_t con, char *line,
     size_t cc_len = 0;
 
     /* Skip the user field. */
-    cc_str = index(con, ':');
+    cc_str = strchr(con, ':');
     if (!cc_str)
 	    return -1;
     cc_str++;
@@ -57,8 +96,8 @@ static int find_line (FILE *infile, security_context_t con, char *line,
     if (!current_line)
 	    return (-1);
 
-    while (!feof (infile)) {
-	if (!fgets(current_line, length, infile)) {
+    while (!feof_unlocked (infile)) {
+	if (!fgets_unlocked(current_line, length, infile)) {
 		free(current_line);
 		return -1;
 	}
@@ -127,6 +166,8 @@ static int list_from_string (char *instr, const char *user,
     ptr = instr;
     while (*ptr && (count < pri_length))
     { 
+        char *cp;
+
         /* Skip leading whitespace */
         while (*ptr && isspace(*ptr))
 		ptr++;
@@ -145,10 +186,10 @@ static int list_from_string (char *instr, const char *user,
 	if (current_context == NULL)
 		return count;
 
-	strcpy (current_context, user);
-	strcat (current_context, ":");
-	strncat (current_context, ptr, length);
-	current_context[current_context_len-1] = '\0';
+       cp = stpcpy (current_context, user);
+        *cp++ = ':';
+       cp = mempcpy (cp, ptr, length);
+        *cp = '\0';
 
 	/* Check the validity of the context, i.e. user->role,
 	   role->domain authorizations. */
@@ -208,54 +249,26 @@ static int get_config_priority (security_context_t fromcon, const char *user,
     FILE *config_file;    /* The configuration file                    */
     char *fname = 0;      /* The name of the user's configuration file */
     size_t fname_len;     /* The length of fname                       */
-    struct passwd pwdbuf, *pwd = &pwdbuf;   
-    char *buf;
     int retval;           /* The return value                          */
-    long buflen;
 
     if (which == USERPRIORITY)
     {
-        /* Get the password structure in order to find the home directory.
-	   Use getpwnam_r to avoid clobbering any existing pwd struct obtained
-	   by the caller. */
-	buflen = sysconf(_SC_GETPW_R_SIZE_MAX);
-	if (buflen < 0)
-		return -1;
-	buf = malloc(buflen);
-	if (!buf)
-		return -1;
-	retval = getpwnam_r (user, pwd, buf, buflen, &pwd );
-	while (retval < 0 && errno == ERANGE) {
-		char *newbuf;
-		errno = 0;
-		buflen *= 2;
-		newbuf = realloc (buf, buflen);
-		if (!newbuf) {
-			free(buf);
-			return -1;
-		}
-		buf = newbuf;
-		retval = getpwnam_r (user, pwd, buf, buflen, &pwd );
-	}
-        if (retval < 0 || !pwd) {
-		free(buf);
-		return -1;
-        }
-        fname_len = strlen (pwd->pw_dir) + 20;
-        fname = malloc (fname_len);
-        if (!fname) 
-        {
-	    free(buf);
-            return -1;
-        }
-        sprintf (fname, "%s%s", pwd->pw_dir, "/.default_contexts");
-        config_file = fopen (fname, "r");
-	free (fname);
-        free(buf);
+	    const char *user_contexts_path = selinux_user_contexts_path();
+	    fname_len = strlen(user_contexts_path) + strlen(user) + 2;
+	    fname = malloc (fname_len);
+	    if (!fname) 
+		    return -1;
+	    retval = snprintf (fname, fname_len, "%s/%s", user_contexts_path, user);
+	    if (retval < 0 || (size_t)retval >= fname_len) {
+		    free(fname);
+		    return -1;
+	    }
+	    config_file = fopen (fname, "r");
+	    free (fname);
     }
     else if (which == SYSTEMPRIORITY)
     {
-        config_file = fopen (_DEFCONTEXT_PATH, "r");
+        config_file = fopen (selinux_default_context_path(), "r");
     }
     else
     {
@@ -290,6 +303,9 @@ static inline int insert (security_context_t *ordered_list, int length, int pos,
     if ((pos < length) && (pos >= 0))
     {
         ordered_list[pos] = strdup(new_item);
+        if(!ordered_list[pos])
+            return ret_val;
+
         ret_val = 0;
     }
 
@@ -382,19 +398,19 @@ static int add_priority_list (security_context_t *ordered_list,
     return ret_val;
 }
 
-int get_failsafe_context(const char* user, 
-			 security_context_t *newcon)
+static int get_failsafe_context(const char* user, 
+				security_context_t *newcon)
 {
 	FILE *fp;
 	char buf[255], *ptr;
 	size_t plen, nlen;
 	int rc;
 
-	fp = fopen(_FAILSAFECONTEXT_PATH, "r");
+	fp = fopen(selinux_failsafe_context_path(), "r");
 	if (!fp)
 		return -1;
 
-	ptr = fgets(buf, sizeof buf, fp);
+	ptr = fgets_unlocked(buf, sizeof buf, fp);
 	fclose(fp);
 
 	if (!ptr)
@@ -406,6 +422,8 @@ int get_failsafe_context(const char* user,
  retry:
 	nlen = strlen(user)+1+plen+1;
 	*newcon = malloc(nlen);
+	if(!(*newcon))
+		return -1;
 	rc = snprintf(*newcon, nlen, "%s:%s", user, ptr);
 	if (rc < 0 || (size_t) rc >= nlen) {
 		free(*newcon);
@@ -555,3 +573,4 @@ failsafe:
 	    rc = 1;
     goto out3;
 }
+hidden_def(get_ordered_context_list)
