@@ -2,15 +2,31 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <string.h>
-#include <selinux/selinux.h>
+#include "selinux_internal.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
 #include <regex.h>
+#include <stdarg.h>
 #include "policy.h"
 
+static void default_printf(const char *fmt, ...) 
+{
+	va_list ap;
+	va_start(ap, fmt);
+	vfprintf(stderr, fmt, ap);
+	va_end(ap);
+}
+
+static void (*myprintf)(const char *fmt, ...) = &default_printf;
+
+void set_matchpathcon_printf(void (*f)(const char *fmt, ...))
+{
+	myprintf = f;
+}
+
 /*
  * A file security context specification.
  */
@@ -89,10 +105,12 @@ static int find_stem_from_spec(const char **buf)
 	}
 	if(num_stems == alloc_stems)
 	{
+		stem_t *tmp_arr;
 		alloc_stems = alloc_stems * 2 + 16;
-		stem_arr = realloc(stem_arr, sizeof(stem_t) * alloc_stems);
-		if(!stem_arr)
+		tmp_arr = realloc(stem_arr, sizeof(stem_t) * alloc_stems);
+		if(!tmp_arr)
 			return -1;
+		stem_arr = tmp_arr;
 	}
 	stem_arr[num_stems].len = stem_len;
 	stem_arr[num_stems].buf = malloc(stem_len + 1);
@@ -189,6 +207,7 @@ static void spec_hasMetaChars(struct spec *spec)
 static int matchpathcon_init(void)
 {
 	FILE *fp;
+	const char *path;
 	char line_buf[BUFSIZ + 1], *buf_p;
 	char *regex, *type, *context;
 	char *anchored_regex;
@@ -196,7 +215,8 @@ static int matchpathcon_init(void)
 	spec_t *spec_copy;
 
 	/* Open the specification file. */
-	if ((fp = fopen(FILECONTEXTS, "r")) == NULL)
+	path = selinux_file_context_path();
+	if ((fp = fopen(path, "r")) == NULL)
 		return -1;
 
 	/* 
@@ -210,12 +230,12 @@ static int matchpathcon_init(void)
 	for (pass = 0; pass < 2; pass++) {
 		lineno = 0;
 		nspec = 0;
-		while (fgets(line_buf, sizeof line_buf, fp)) {
+		while (fgets_unlocked(line_buf, sizeof line_buf, fp)) {
 			lineno++;
 			len = strlen(line_buf);
 			if (line_buf[len - 1] != '\n') {
-				errno = EINVAL;
-				return -1;
+				myprintf("%s:  line %d is too long, would be truncated, skipping\n", path, lineno); 
+				continue;
 			}
 			line_buf[len - 1] = 0;
 			buf_p = line_buf;
@@ -228,8 +248,8 @@ static int matchpathcon_init(void)
 			    sscanf(line_buf, "%as %as %as", &regex, &type,
 				   &context);
 			if (items < 2) {
-				errno = EINVAL;
-				return -1;
+				myprintf("%s:  line %d is missing fields\n, skipping", path, lineno); 
+				continue;
 			} else if (items == 2) {
 				/* The type field is optional. */
 				free(context);
@@ -240,15 +260,20 @@ static int matchpathcon_init(void)
 			if (pass == 1) {
 				/* On the second pass, compile and store the specification in spec. */
 				const char *reg_buf = regex;
+				char *cp;
 				spec_arr[nspec].stem_id = find_stem_from_spec(&reg_buf);
 				spec_arr[nspec].regex_str = regex;
 
 				/* Anchor the regular expression. */
 				len = strlen(reg_buf);
-				anchored_regex = malloc(len + 3);
+				cp = anchored_regex = malloc(len + 3);
 				if (!anchored_regex)
 					return -1;
-				sprintf(anchored_regex, "^%s$", reg_buf);
+				/* Create ^...$ regexp.  */
+				*cp++ = '^';
+				cp = mempcpy(cp, reg_buf, len);
+				*cp++ = '$';
+				*cp = '\0';
 
 				/* Compile the regular expression. */
 				regerr =
@@ -257,8 +282,8 @@ static int matchpathcon_init(void)
 					    REG_EXTENDED | REG_NOSUB);
 				free(anchored_regex);
 				if (regerr < 0) {
-					errno = EINVAL;
-					return -1;
+					myprintf("%s:  line %d has invalid regex %s\n", path, lineno, anchored_regex); 
+					continue;
 				}
 
 				/* Convert the type string to a mode format */
@@ -268,8 +293,8 @@ static int matchpathcon_init(void)
 					goto skip_type;
 				len = strlen(type);
 				if (type[0] != '-' || len != 2) {
-					errno = EINVAL;
-					return -1;
+					myprintf("%s:  line %d has invalid file type %s\n", path, lineno, type); 
+					continue;
 				}
 				switch (type[1]) {
 				case 'b':
@@ -294,8 +319,8 @@ static int matchpathcon_init(void)
 					spec_arr[nspec].mode = S_IFREG;
 					break;
 				default:
-					errno = EINVAL;
-					return -1;
+					myprintf("%s:  line %d has invalid file type %s\n", path, lineno, type); 
+					continue;
 				}
 
 			      skip_type:
@@ -304,8 +329,8 @@ static int matchpathcon_init(void)
 
 				if (strcmp(context, "<<none>>")) {
 					if (security_check_context(context) < 0 && errno != ENOENT) {
-						errno = EINVAL;
-						return -1;
+						myprintf("%s:  line %d has invalid context %s\n", path, lineno, context); 
+						continue;
 					}
 				}
 
@@ -329,7 +354,7 @@ static int matchpathcon_init(void)
 			if ((spec_arr = malloc(sizeof(spec_t) * nspec)) ==
 			    NULL)
 				return -1;
-			bzero(spec_arr, sizeof(spec_t) * nspec);
+			memset(spec_arr, '\0', sizeof(spec_t) * nspec);
 			rewind(fp);
 		}
 	}
